__author__ = 'Martin'

import math

"""
BUILDING A TRUTH TABLE

You will probably want to put the code for your tables in a separate file (recommended)
so right click on Logic in the project tree and do New > Python File call it 'my_tables'
for example. It opens in the editor. Unless you have a very small monitor it will be
more convenient to have the file in a separate window so left mouse down on the tab with
the file name and drag it out of this widow.

1) IMPORT THE REQUIRED CLASSES

type in

from truth_tables import TruthTable, tt

2) CREATE A NEW TRUTH TABLE

Think of a name and type it in, any name other than 'tt'

table_1 = TruthTable(2)

(2) is the number of variables you want to use. By default any number up to 10 can be
used but 10 variables will get 2^10 rows in the table which is quite a lot. Default
variable names are 'p', 'q', etc.

      0    1    2    3    4    5    6    7    8    9    10
    ['p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z']

The names are mapped to the integers shown by the tt class. Their only significance
relates to what appears in output. To change the names type in:

table_1.vnames = ['A', 'B'] # as many names as you like, any single characters

The class tt will not recognize these names but no problem A == 0, B == 1

3) ADD A COLUMN

I am going to use the app. to test one of De Morgan's laws. A truth table for any substantial
expression can probably benefit from some intermediate expressions particularly if you
are going to use the blank table facility to practice filling out tables by hand.
The columns appear in the order they are added to the table so a start could be an
'A and B' column.

First define a function that will compute the expression. The function name, parameter
and variable names are entirely arbitrary: use any names you like:

def A_and_B(vars):
    A = vars[0]; B = vars[1]
    return A and B

vars is a list of parameters passed to the function with indices
                              corresponding to the variables A and B as discussed above
Local variables A and B are the current values got from this list by indexing into it.
The return is the result of the expression (A and B)

Next create an object of type TruthTable.Column including a title for the column and
the function just defined:

col_object = TruthTable.Column('A and B', A_and_B)

Finally add the column to the table:

table_1.add_column(col_object)

4) TEST ALL IS WELL

Add the line:

table_1.print_table()

Run the file by right clicking on it in the edit window and selecting Run 'my_tables'
from the context menu. Output appears in the Run pane of the main window. It should look
like this:

| A | B | A and B |
+---+---+---------+-
| T | T |    T    |
| T | F |    F    |
| F | T |    F    |
| F | F |    F    |

Process finished with exit code 0

5) ADD THE OTHER TWO COLUMNS

def not_A_and_B(vars):
    A = vars[0]; B = vars[1]
    return not (A and B)
col_object = TruthTable.Column('not(A and B)', not_A_and_B)
table_1.add_column(col_object)

def notA_or_notB(vars):
    A = vars[0]; B = vars[1]
    return (not A) or (not B)
col_object = TruthTable.Column('(not A) or (not B)', notA_or_notB)
table_1.add_column(col_object)

6) PRINT THE COMPLETED TABLE

To display a complete table add the line

table_1.print_table()

To display a blank table

table_1.print_table(blank = True)

Run 'my_tables'

Any problems - the completed code is in 'Martins_tables.py'

Use of the tt class to map the default variable names is shown under
if __name__ == '__main__':
below.

"""


class tt():
    p = 0; q = 1; r = 2; s = 3; t = 4; u = 5;
    v = 6; w = 7; x = 8; y = 9; z = 10

def truth(i):
    if i:
        return 'T'
    else:
        return 'F'

class _TruthTable():
    class Column:
        def __init__(self, title, expr):
            self.title = title
            self.expr = expr
            self.field_width = len(title) + 1

class TruthTable(_TruthTable):

    def __init__(self, num_variables):
        self.num_variables = num_variables
        self.var_cols = 2 ** num_variables
                      # 0    1    2    3    4    5    6    7    8    9    10
        self.vnames = ['p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z']
        self.exprs = []

    def add_column(self, expr: _TruthTable.Column):
        self.exprs.append(expr)

    def print_table(self, blank = False):
        # build a string holding the column titles and another cross line to go under
        n_expr = len(self.exprs)
        n_vars = self.num_variables
        titles = '|'
        cross_line = '+'
        for i in range(n_vars):
            titles += ' ' + self.vnames[i] + ' |'
            cross_line += '---+'
        for i in range(n_expr):
            titles += ' ' + self.exprs[i].title + ' |'
            cross_line += '-'*(self.exprs[i].field_width) + '-+'
        # top_line = '+' + '-' * (len(cross_line) - 2) + '+'
        # print(top_line)
        print(titles)
        print(cross_line)

        # compute the maximum value with all the variables set True
        max_vals = self.num_variables ** 2 - 1
        # start with all the variables False
        bits = max_vals
        while bits >= 0:
            # convert the current value of bits to a list of ones and zeros
            bit_lst = [bits >> i & 1 for i in range(n_vars-1, -1, -1)]

            # build a string to represent the corresponding row
            row = '| '
            for i in range(n_vars):
                row += truth(bit_lst[i]) + ' | '
            for i in range(n_expr):
                cur_expr = self.exprs[i]
                ldrs = cur_expr.field_width // 2 - 1
                if blank: bres = ' '
                else: bres = truth(cur_expr.expr(bit_lst))
                row += ' ' * ldrs + bres + ' ' * (cur_expr.field_width - ldrs - 1) + '| '
            print(row)
            bits -= 1
        print()




if __name__ == '__main__':
    print()

    t_table = TruthTable(2)

    def _not(v):
        return not v[tt.p]
    not_expr = TruthTable.Column('not p', _not)
    t_table.add_column(not_expr)

    def and_exp(v):
        return v[tt.p] and v[tt.q]
    and_expr = TruthTable.Column('p and q', and_exp)
    t_table.add_column(and_expr)

    def or_exp(v):
        return v[tt.p] or v[tt.q]
    or_expr = TruthTable.Column('p or q', or_exp)
    t_table.add_column(or_expr)

    def xor_exp(v):
        return v[tt.p] ^ v[tt.q]
    xor_expr = TruthTable.Column('p xor q', xor_exp)
    t_table.add_column(xor_expr)

    def impl_exp(v):
        p = v[tt.p]; q = v[tt.q]
        return not p or p and q
    impl_expr = TruthTable.Column('p => q', impl_exp)
    t_table.add_column(impl_expr)

    def imp_exp(v):
        p = v[tt.p]; q = v[tt.q]
        return ~p &1|p& q
        #return (not p )|p& q
    imp_expr = TruthTable.Column('p -> q', imp_exp)
    t_table.add_column(imp_expr)

    t_table.print_table()
    print()

    t_table = TruthTable(4)

    def q_and_r_exp(v):
        q = v[tt.q]; r = v[tt.r]
        return q and r
    q_and_r_expr = TruthTable.Column('q and r', q_and_r_exp)
    t_table.add_column(q_and_r_expr)

    def p_or_q_and_r_or_s_exp(v):
        p = v[tt.p]; q = v[tt.q]; r = v[tt.r]; s = v[tt.s]
        return p or q and r or s
    pqrs_expr = TruthTable.Column('p or q and r or s', p_or_q_and_r_or_s_exp)
    t_table.add_column(pqrs_expr)

    def p_or_q_exp(v):
        p = v[tt.p]; q = v[tt.q]
        return p or q
    p_or_q_expr = TruthTable.Column('p or q', p_or_q_exp)
    t_table.add_column(p_or_q_expr)

    def r_or_s_exp(v):
        r = v[tt.r]; s = v[tt.s]
        return r or s
    r_or_s_expr = TruthTable.Column('r or s', r_or_s_exp)
    t_table.add_column(r_or_s_expr)

    def porq_and_rors_exp(v):
        p = v[tt.p]; q = v[tt.q]; r = v[tt.r]; s = v[tt.s]
        return (p or q) and (r or s)
    pqrsb_expr = TruthTable.Column('(p or q) and (r or s)', porq_and_rors_exp)
    t_table.add_column(pqrsb_expr)

    t_table.print_table(blank = True)






